<?php

namespace App\Controller\OgRepository\Webhook;

use App\Entity\Image;
use App\Entity\Trace;
use App\Model\ImageStatus;
use App\Model\TraceStatus;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Attribute\AsController;
use Symfony\Component\Routing\Annotation\Route;

#[AsController]
class ResponseController extends AbstractController
{
    public function __construct(
        protected readonly EntityManagerInterface $entityManager
    )
    {
    }

    #[Route('/og-repository/webhook', name: 'og_repository_webhook', methods: ['POST'])]
    public function repositoryWebhook(Request $request): JsonResponse
    {
        $data = json_decode($request->getContent(), true);

        $trace = $this->entityManager->getRepository(Trace::class)->findOneBy(['jobId' => $data['job_id']]);
        $imageUuid = $trace->getInput()['imageUuid'];

        $image = $this->entityManager->getRepository(Image::class)->findOneBy(['uuid' => $imageUuid]);

        if ($image === null) {
            $trace->setStatus(TraceStatus::FAILED);
            $trace->setFinishedAt(new \DateTime());
            $trace->setOutput('Image not found');
            $this->entityManager->persist($trace);
            $this->entityManager->flush();

            return new JsonResponse(['message' => 'Image not found'], Response::HTTP_NOT_FOUND);
        }

        $image->setImageFullsum($data['image_id']);
        $image->setStatus(ImageStatus::SUCCESS);
        $this->entityManager->persist($image);

        $trace->setStatus(TraceStatus::SUCCESS);
        $trace->setFinishedAt(new \DateTime());

        $this->entityManager->persist($trace);
        $this->entityManager->flush();

        return new JsonResponse($data, Response::HTTP_OK);

    }
}