<?php

namespace App\Dto\Output;

use ApiPlatform\Metadata\ApiProperty;
use ApiPlatform\Metadata\Get;
use App\Entity\Client;
use App\Entity\OrganizationalUnit;
use App\Entity\Subnet;
use Symfony\Component\Serializer\Annotation\Groups;

#[Get(shortName: 'Subnet')]
final class SubnetOutput extends AbstractOutput
{
    #[Groups(['subnet:read'])]
    public string $name;

    #[Groups(['subnet:read'])]
    public string $netmask;

    #[Groups(['subnet:read'])]
    public string $ipAddress;

    #[Groups(['subnet:read'])]
    public string $nextServer;

    #[Groups(['subnet:read'])]
    public string $bootFileName;

    #[Groups(['subnet:read'])]
    public array $clients;

    #[Groups(['subnet:read'])]
    public ?bool $synchronized = false;

    #[Groups(['subnet:read'])]
    public ?int $serverId;

    #[Groups(['subnet:read'])]
    public \DateTime $createdAt;

    #[Groups(['subnet:read'])]
    public ?string $createdBy = null;

    public function __construct(Subnet $subnet)
    {
        parent::__construct($subnet);

        $this->name = $subnet->getName();
        $this->netmask = $subnet->getNetmask();
        $this->ipAddress = $subnet->getIpAddress();
        $this->nextServer = $subnet->getNextServer();
        $this->bootFileName = $subnet->getBootFileName();
        $this->synchronized = $subnet->isSynchronized();
        $this->serverId = $subnet->getServerId();

        $this->clients = $subnet->getClients()->map(
            fn(Client $client) => new ClientOutput($client)
        )->toArray();

        $this->createdAt = $subnet->getCreatedAt();
        $this->createdBy = $subnet->getCreatedBy();
    }
}